import * as jwt from "jsonwebtoken";
import { Action } from "routing-controllers";
import { getConnectionManager } from 'typeorm';
import { config } from './config';
import { IllegalJWTError, NoPermissionError, UserNonexistantOrRefreshtokenInvalidError } from './errors/AuthError';
import { User } from './models/entities/User';

const authchecker = async (action: Action, permissions: string[] | string) => {
    let required_permissions = undefined;
    if (typeof permissions === "string") {
        required_permissions = [permissions]
    } else {
        required_permissions = permissions
    }

    const provided_token = action.request.headers["authorization"].replace("Bearer ", "");
    let jwtPayload = undefined
    try {
        jwtPayload = <any>jwt.verify(provided_token, config.jwt_secret);
    } catch (error) {
        throw new IllegalJWTError()
    }
    const user = await getConnectionManager().get().getRepository(User).findOne({ id: jwtPayload["userdetails"]["id"], refreshTokenCount: jwtPayload["userdetails"]["refreshTokenCount"] }, { relations: ['permissions'] })
    if (!user) { throw new UserNonexistantOrRefreshtokenInvalidError() }
    if (!jwtPayload.permissions) { throw new NoPermissionError(); }

    action.response.local = {}
    action.response.local.jwtPayload = jwtPayload.permissions
    required_permissions.forEach(r => {
        const permission_key = r.split(":")[0]
        const actual_accesslevel_for_permission = jwtPayload.permissions[permission_key]
        const permission_access_level = r.split(":")[1]
        if (actual_accesslevel_for_permission.includes(permission_access_level)) {
            return true;
        } else {
            throw new NoPermissionError()
        }
    });
    try {
        jwt.verify(provided_token, config.jwt_secret);
        return true
    } catch (error) {
        return false
    }
}
export default authchecker